var Benchmark     = require('benchmark');
var BinSearchTree    = require('../src/Tree');
var Bstree           = require('bstree');
var BinarySearchTree = require('binary-search-tree').BinarySearchTree;

var hash = Object.create(null);
var bstree           = new Bstree(function(k1, k2) { return k1 < k2 });
var binSearchTree    = new BinSearchTree(function(k1, k2) { return k1 < k2 });
var binSearchTree1   = new BinSearchTree(function(k1, k2) { return k1 < k2 });
var binSearchTree1p  = new BinSearchTree(function(k1, k2) { return k1 < k2 });
var binSearchTree2   = new BinSearchTree(function(k1, k2) { return k1 < k2 });
var binarySearchTree = new BinarySearchTree({

    compareKeys: function(k1, k2) {
        if (k1 < k2) {
            return -1;
        }
        if (k1 > k2) {
            return 1;
        }
        return 0;
    }
});
var array = [];

binSearchTree.trav  = algorithm4;
binSearchTree1.trav = algorithm4a;
//binSearchTree1p.trav = algorithm4p;
//binSearchTree2._traverse = algorithm4;

for (var i = 0; i < 100000; ++i) {
    var val = Math.floor(Math.random()* 1000000);

    if (hash[val]) {
        continue;
    }

    hash[val] = true;

    array.push(val);
    binSearchTree.set(val);
    binSearchTree1.set(val);
    //binSearchTree1p.set(val);
    //binSearchTree2.set(val);
    //bstree.add(val);
    binarySearchTree.insert(val);
}

hash = null;

var count, count1, count2, count3, count4, count5;

new Benchmark.Suite()
    .add('bin-search-tree', function() {
        count = 0;
        binSearchTree.trav(function() {
            count += 1;
        });
    })
    .add('bin-search-tree1', function() {
        count1 = 0;
        binSearchTree1.trav(function() {
            count1 += 1;
        });
    })
    //.add('array', function() {
    //    count2 = 0;
    //    array.sort(function(v1, v2) { if (v1 < v2) return -1; if (v1 > v2) return 1; return 0});
    //    array.forEach(function() {
    //        count2 += 1;
    //    });
    //})
    //.add('bin-search-tree2', function() {
    //    count3 = 0;
    //    binSearchTree2.forEach(function() {
    //        count3 += 1;
    //    });
    //})
    //.add('bstree', function() {
    //    count4 = 0;
    //    bstree.forEach(function(value) {
    //        count4 += 1;
    //    });
    //})
    .add('binary-search-tree', function() {
        count5 = 0;
        binarySearchTree.executeOnEveryNode(function() {
            count5 += 1;
        });
    })
    .on('complete', function() {
        console.log(this[0].toString(), count);
        console.log(this[1].toString(), count1);
        //console.log(this[2].toString(), count2);
        //console.log(this[3].toString(), count3);
        //console.log(this[4].toString(), count4);
        console.log(this[2].toString(), count5);

        console.log('Fastest is ' + this.filter('fastest').pluck('name'));
    })
    .on('error', function() {
        console.log(arguments);
    })
    .run();


function traverse2(callback) {
    "use strict";

    var node = this._min;
    var last = node;

    callback(node);

    node = node.r || node.p;

    while (node) {
        if (last === node.p) {
            if (node.l) {
                last = node;
                node = node.l;
                continue;
            }

            callback(node);

            if (node.r) {
                last = node;
                node = node.r;
                continue;
            }
        }
        else if (last === node.l) {
            callback(node);

            if (node.r) {
                last = node;
                node = node.r;
                continue;
            }
        }

        last = node;
        node = node.p;
    }
}

function algorithmLong(callback) {
    var stack = [];
    var node  = this._min;

    if (!node) {
        return;
    }

    callback(node);

    node = node.r || node.p;

    while (node) {
        if (!up && node.l) {
            node = node.l;
            continue;
        }

        callback(node);

        if (node.r) {
            stack.push(node);
            up   = false;
            node = node.r;
            continue;
        }

        if (!up && node.p.l === node) {
            up   = true;
            node = node.p;
            continue;
        }

        up   = true;
        node = stack.pop().p;
    }
}

function algorithm1(callback) {
    'use strict';

    return inNode(this._root, callback);
}

function inNode(node, callback) {
    'use strict';

    if (node.l) inNode(node.l, callback);
    callback(node);
    if (node.r) inNode(node.r, callback);
}

function algorithm2(callback) {
    var node = this._root;
    var stack = [];

    var up = false;

    while (node) {

        if (up) {
            callback(node);
            node = node.r ? (up = false, node.r)  : stack.pop();
            continue;
        }

        if (node.l) {
            stack.push(node);
            node = node.l;
            continue;
        }

        callback(node);

        if (node.r) {
            node = node.r;
            continue;
        }

        up = true;
        node = stack.pop();
    }
}

function algorithm4(callback) {
    "use strict";

    var node  = this._root;
    var stack = new Array(50);
    var index = 0;

    while (node['l']) {
        stack[index++] = node;
        node = node['l'];
    }

    callback(node);

    while (node['r']) {
        node = node['r'];

        while (node['l']) {
            stack[index++] = node;
            node = node['l'];
        }

        callback(node);
    }

    while (index) {
        node = stack[--index];

        callback(node);

        while (node['r']) {
            node = node['r'];

            while (node['l']) {
                stack[index++] = node;
                node = node['l'];
            }

            callback(node);
        }
    }
}

function algorithm4a(callback) {
    "use strict";

    var node  = this._root;
    var stack = new Array(50);
    var index = 0;

    main: while (node) {
        if (node['l']) {
            stack[index++] = node;
            node = node['l'];
            continue;
        }

        callback(node);

        if (node['r']) {
            node = node['r'];
            continue;
        }


        while (index) {
            node = stack[--index];

            callback(node);

            if (node['r']) {
                node = node['r'];
                continue main;
            }
        }

        return;
    }
}

function algorithm4p(callback) {
    "use asm";

    var node  = this._root;
    var stack = new Array(30);
    var index = 0;

    main: while (node) {
        if (node['l']) {
            stack[index++] = node;
            node = node['l'];
            continue;
        }

        callback(node);

        if (node['r']) {
            node = node['r'];
            continue;
        }


        while (index) {
            node = stack[--index];

            callback(node);

            if (node['r']) {
                node = node['r'];
                continue main;
            }
        }

        return;
    }
}

//function algorithm4p(callback) {
//    "use strict";
//
//    var node  = this._root;
//    var stack = [];
//
//    while (node) {
//        if (node['l']) {
//            stack.push(node);
//            node = node['l'];
//            continue;
//        }
//
//        callback(node);
//
//        if (node['r']) {
//            node = node['r'];
//            continue;
//        }
//
//        node = stack.pop();
//
//        while (node) {
//            callback(node);
//
//            if (node['r']) {
//                node = node['r'];
//                break;
//            }
//
//            node = stack.pop();
//        }
//    }
//}

function algorithm3(callback) {
    'use strict';

    var node  = this._min;
    var stack = [];

    main: while (node) {

        if (node.l) {
            node = node.l;
            continue;
        }

        callback(node);

        if (node.r) {
            stack.push(node);
            node = node.r;
            continue;
        }

        var parent = node.p;

        while (parent) {
            if (node === parent.l) {
                callback(parent);

                if (parent.r) {
                    stack.push(parent);
                    node = parent.r;
                    continue main;
                }
                node = parent;
            }
            else {
                node = stack.pop();
            }
            parent = node.p;
        }

        return;
    }
}


//function algorithm4(callback) {
//    var node  = this._min;
//    var stack = [];
//    var up = true;
//
//    while (node) {
//
//        if (!up && node.l) {
//            node = node.l;
//            continue;
//        }
//
//        callback(node);
//
//        if (node.r) {
//            stack.push(node);
//            up   = false;
//            node = node.r;
//            continue;
//        }
//
//        var parent = node.p;
//        up = true;
//
//        if (node === parent.l) {
//            node = parent;
//            continue;
//        }
//
//        parent = stack.pop();
//        node   = parent.p;
//    }
//}
//
//function produce(callback) {
//    var min   = $min;
//    var $node = min;
//
//    var stack = new Array(50);
//    var index = 0;
//
//    var newStack = new Array(50);
//    var newIndex = 0;
//
//    var newNode = null;
//    var last    = null;
//
//    do {
//        newNode = callback();
//
//        if (last) {
//            if ($node.l.k === last.k) {
//                last['p'] = newNode;
//                newNode['l'] = last;
//            }
//
//            if (newIndex) {
//                var middle = newStack[newIndex];
//
//                if ($node.p.k === middle.k) {
//                    middle['r'] = newNode;
//                    newNode['p'] = middle;
//                }
//
//                if ($node.l.k === middle.k) {
//                    middle['p'] = newNode;
//                    newNode['l'] = middle;
//                    --newIndex;
//                }
//            }
//        }
//
//        last = newNode;
//
//        while ($node['r']) {
//            $node = $node['r'];
//
//            while ($node['l']) {
//                if (last) {
//                    newStack[newIndex++] = last;
//                    last = null;
//                }
//                stack[index++] = $node;
//                $node = $node['l'];
//            }
//
//            newNode = callback();
//
//            if (last) {
//                last['r']    = newNode;
//                newNode['p'] = last;
//            }
//
//            last = newNode;
//        }
//
//    } while ($node = (index ? stack[--index] : min = min['p']));
//}


function produce(callback) {
    var min   = $min;
    var $node = min;

    var stack = new Array(50);
    var index = 0;

    var newStack  = new Array(50);
    var newIndex  = 0;
    var lastNew   = null;
    var lastRight = null;

    var newNode = null;
    var last    = null;

    do {
        newNode = callback();

        if (last) {
            newNode['l'] = last;
            last['p'] = newNode;
        }

        if (lastNew) {
            if (lastNew['k'] === $node['p']['k']) {
                lastNew['r'] = newNode;
                newNode['p'] = lastNew;
            }
            if (lastNew['k'] === $node['l']['k']) {
                lastNew['p'] = newNode;
                newNode['l'] = lastNew;
                lastNew = newStack[--newIndex];
            }
        }

        last = lastRight = newNode;

        while ($node['r']) {
            $node = $node['r'];

            while ($node['l']) {
                if (last) {
                    lastNew = newStack[newIndex++] = last;
                    last = null;
                }
                stack[index++] = $node;
                $node = $node['l'];
            }

            newNode = callback();

            if (last) {
                lastRight['r'] = newNode;
                newNode['p'] = lastRight;
            }
            else {
                last = newNode;
            }

            lastRight = newNode;
        }

    } while ($node = (index ? stack[--index] : min = min['p']));
}

//function algorithm4(callback) {
//    var node  = this._min;
//    var stack = [];
//    var up = true;
//
//    while (node) {
//
//        if (!up && node.l) {
//            node = node.l;
//            continue;
//        }
//
//        callback(node);
//
//        if (node.r) {
//            stack.push(node);
//            up   = false;
//            node = node.r;
//            continue;
//        }
//
//        var parent = node.p;
//        up = true;
//
//        if (node === parent.l) {
//            node = parent;
//            continue;
//        }
//
//        parent = stack.pop();
//        node   = parent.p;
//    }
//}

//
//function algorithm4(callback) {
//    var min = this._min;
//    var max = this._max;
//
//    var last, stack = [];
//
//    var node = min;
//
//    main: while (node) {
//
//        if (node.r) {
//            stack.push(node);
//            node = node.r;
//            continue;
//        }
//
//        callback(node);
//
//        if (node.r) {
//            last = node;
//            node = node.r;
//            continue;
//        }
//
//        var p = node.p;
//
//        if (p.r === node) {
//            p = stack.pop();
//
//        }
//
//        callback(p);
//
//        if (p.r) {
//            node = p.r;
//            continue;
//        }
//
//        while (node.p) {
//            var par = node.p;
//
//            if (par.l === node) {
//                callback(par);
//                if (par.r) {
//                    node = par.r;
//                    continue main;
//                }
//            }
//            node = par;
//        }
//
//        node = node.p;
//    }
//}