"use strict";

var Benchmark     = require('benchmark');
var BinSearchTree    = require('../src/Tree');
var BinarySearchTree = require('binary-search-tree').BinarySearchTree;

var hash = Object.create(null);
var binSearchTree1   = new BinSearchTree(function(k1, k2) { return k1 < k2 });
var binSearchTree2   = new BinSearchTree(function(k1, k2) { return k1 < k2 });
var binarySearchTree = new BinarySearchTree({

    compareKeys: function(k1, k2) {
        if (k1 < k2) {
            return -1;
        }
        if (k1 > k2) {
            return 1;
        }
        return 0;
    }
});

for (var i = 0; i < 1000000; ++i) {
    var val = Math.floor(Math.random()* 1000000);

    if (hash[val]) {
        continue;
    }

    hash[val] = true;
    binSearchTree1.set(val);
    binSearchTree2.set(val);
    binarySearchTree.insert(val);
}

var count1, count2, count3;


new Benchmark.Suite()
    .add('binary-search-tree', function() {
        count1 = 0;
        binarySearchTree.executeOnEveryNode(function() {
            count1 += 1;
        });
    })
    //.add('bin-search-tree1', function() {
    //    count2 = 0;
    //    traverse2(binSearchTree1._root, function() {
    //        count2 += 1;
    //    });
    //})
    .add('bin-search-tree2', function() {
        count3 = 0;
        traverse4(binSearchTree2._min, function() {
            count3 += 1;
        });
    })
    .on('complete', function() {
        console.log(this[0].toString(), count1);
        //console.log(this[0].toString(), count2);
        console.log(this[1].toString(), count3);

        console.log('Fastest is ' + this.filter('fastest').pluck('name'));
    })
    .on('error', function() {
        console.log(arguments);
    })
    .run();

function traverse1(node, callback) {
    var stack = new Array(50);
    var index = 0;

    while (node['l']) {
        stack[index++] = node;
        node = node['l'];
    }

    callback(node);

    while (node['r']) {
        node = node['r'];

        while (node['l']) {
            stack[index++] = node;
            node = node['l'];
        }

        callback(node);
    }

    while (index) {
        node = stack[--index];

        callback(node);

        while (node['r']) {
            node = node['r'];

            while (node['l']) {
                stack[index++] = node;
                node = node['l'];
            }

            callback(node);
        }
    }
}


function traverse2(node, callback) {
    var stack = new Array(50);
    var index = 0;

    main: while (node) {
        if (node['l']) {
            stack[index++] = node;
            node = node['l'];
            continue;
        }

        callback(node);

        if (node['r']) {
            node = node['r'];
            continue;
        }


        while (index) {
            node = stack[--index];

            callback(node);

            if (node['r']) {
                node = node['r'];
                continue main;
            }
        }

        return;
    }
}

function traverse3(min, callback) {
    var node  = min;
    var stack = new Array(50);
    var index = 0;

    do {

        callback(node);

        while (node['r']) {
            node = node['r'];

            while (node['l']) {
                stack[index++] = node;
                node = node['l'];
            }

            callback(node);
        }

        while (index) {
            node = stack[--index];

            callback(node);

            while (node['r']) {
                node = node['r'];

                while (node['l']) {
                    stack[index++] = node;
                    node = node['l'];
                }

                callback(node);
            }
        }
    } while (node = min = min['p']);
}


function traverse4(min, callback) {
    var node  = min;
    var stack = new Array(50);
    var index = 0;

    do {
        callback(node);

        while (node['r']) {
            node = node['r'];

            while (node['l']) {
                stack[index++] = node;
                node = node['l'];
            }

            callback(node);
        }
    } while (node = (index ? stack[--index] : min = min['p']));
}

function traverse6(node, callback) {
    var stack = new Array(50);
    var index = 0;


    code1();

    callback(node);

    code2();

    while (index) {
        node = stack[--index];

        callback(node);

        code2();
    }

    function code1() {
        while (node['l']) {
            stack[index++] = node;
            node = node['l'];
        }
    }

    function code2() {
        while (node['r']) {
            node = node['r'];

            code1();

            callback(node);
        }
    }
    //
    //code1();
    //
    ////do {
    ////    code2
    ////} while (index && (node = stack[--index]));
    //code2();
    //
    //while (index) {
    //    node = stack[--index];
    //
    //    code2();
    //}
    //
    //function code1() {
    //    while (node['l']) {
    //        stack[index++] = node;
    //        node = node['l'];
    //    }
    //
    //    callback(node);
    //}
    //
    //function code2() {
    //    callback(node);
    //
    //    while (node['r']) {
    //        node = node['r'];
    //
    //        code1();
    //    }
    //}
}