"use strict";

var Benchmark = require('benchmark');
var BinSearchTree    = require('../dist/Tree');
var Bstree           = require('bstree');
var BinarySearchTree = require('binary-search-tree').BinarySearchTree;

var hash = Object.create(null);
var binSearchTree    = new BinSearchTree(function(k1, k2) { return k1 < k2 });
var bstree           = new Bstree(function(k1, k2) { return k1 < k2 });
var binarySearchTree = new BinarySearchTree({

    compareKeys: function(k1, k2) {
        if (k1 < k2) {
            return -1;
        }
        if (k1 > k2) {
            return 1;
        }
        return 0;
    }
});

for (var i = 0; i < 1000000; ++i) {
    var val = Math.floor(Math.random()* 1000000);

    if (hash[val]) {
        continue;
    }

    hash[val] = true;
    bstree.add(val);
    binSearchTree.set(val);
    binarySearchTree.insert(val);
}

var count1, count2, count3;


new Benchmark.Suite()
    .add('binary-search-tree', function() {
        count1 = 0;
        binarySearchTree.executeOnEveryNode(function() {
            count1 += 1;
        });
    })
    .add('bstree', function() {
        count2 = 0;
        bstree.forEach(function() {
            count2 += 1;
        });
    })
    .add('bin-search-tree', function() {
        count3 = 0;
        binSearchTree.forEach(function() {
            count3 += 1;
        });
    })
    .on('complete', function() {
        console.log(this[0].toString(), count1);
        console.log(this[1].toString(), count2);
        console.log(this[2].toString(), count3);

        console.log('Fastest is ' + this.filter('fastest').pluck('name'));
    })
    .on('error', function() {
        console.log(arguments);
    })
    .run();